"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var utilities_1 = require("graphql/utilities");
var prisma_datamodel_1 = require("prisma-datamodel");
var generator_1 = require("../../src/generator");
var fs = require("fs");
var path = require("path");
function blackBoxTest(name, databaseType, filePrefix) {
    var generator = generator_1.default.create(databaseType);
    var modelPath = path.join(__dirname, "cases/" + name + "/model_" + filePrefix + ".graphql");
    var prismaPath = path.join(__dirname, "cases/" + name + "/" + filePrefix + ".graphql");
    expect(fs.existsSync(modelPath));
    expect(fs.existsSync(prismaPath));
    var model = fs.readFileSync(modelPath, { encoding: 'UTF-8' });
    var prisma = fs.readFileSync(prismaPath, { encoding: 'UTF-8' });
    var types = prisma_datamodel_1.Parser.create(databaseType).parseFromSchemaString(model).types;
    var ourSchema = generator.schema.generate(types, {});
    var ourPrintedSchema = utilities_1.printSchema(ourSchema);
    // Write a copy of the generated schema to the FS, for debugging
    fs.writeFileSync(path.join(__dirname, "cases/" + name + "/generated_" + databaseType + ".graphql"), ourPrintedSchema, { encoding: 'UTF-8' });
    // Check if our schema equals the prisma schema.
    var prismaSchema = utilities_1.buildSchema(prisma);
    prisma_datamodel_1.AstTools.assertTypesEqual(prismaSchema, ourSchema, name + "/" + databaseType);
    // Check if we can parse the schema we build (e.g. it's syntactically valid).
    graphql_1.parse(ourPrintedSchema);
}
exports.default = blackBoxTest;
var testNames = fs.readdirSync(path.join(__dirname, 'cases'));
var _loop_1 = function (testName) {
    test("Generates schema for " + testName + "/relational correctly", function () {
        blackBoxTest(testName, prisma_datamodel_1.DatabaseType.postgres, 'relational');
    });
    test("Generates schema for " + testName + "/document correctly", function () {
        blackBoxTest(testName, prisma_datamodel_1.DatabaseType.mongo, 'document');
    });
};
for (var _i = 0, testNames_1 = testNames; _i < testNames_1.length; _i++) {
    var testName = testNames_1[_i];
    _loop_1(testName);
}
//# sourceMappingURL=blackBoxTest.js.map